/* ************************************************************************** */
// File       : pixelformat.h
// Programmer : Tobin Schwaiger-Hastanan
// Copyright (c) 1997-1999 Tobin Schwaiger-Hastanan
// Notes      : 
/* ************************************************************************** */

#ifndef _PIXELFORMAT_DAWT_H_INCLUDED_
#define _PIXELFORMAT_DAWT_H_INCLUDED_

/*
required type definitions - 

typedef unsigned char uchar;
typedef unsigned long uint32;
*/

/* ************************************************************************** */
// CColor class
/* ************************************************************************** */
class CColor
{
public:
	union
	{
		float c[4];
		struct
		{
	        float red;
	        float green;
	        float blue;
	        float alpha;
		};
	};
public:
	CColor( float Red = 0.0f, float Green = 0.0f, float Blue = 0.0f, float Alpha = 0.0f )
	{
		red   = Red;
		green = Green;
		blue  = Blue;
		alpha = Alpha;
	}

	float GetRed()   const { return red;   }
	float GetGreen() const { return green; }
	float GetBlue()  const { return blue;  }
	float GetAlpha() const { return alpha; }
	
	void SetRed( float Red )     { red   = Red;   }
	void SetGreen( float Green ) { green = Green; }
	void SetBlue( float Blue )   { blue  = Blue;  }
	void SetAlpha( float Alpha ) { alpha = Alpha; }
};

/* ************************************************************************** */
// CPixelFormat class
/* ************************************************************************** */
class CPixelFormat
{
public:
	CPixelFormat( int BPP = 32, 			 uint32 RedMask  = 0xFF0000,
				  uint32 GreenMask = 0xFF00, uint32 BlueMask = 0xFF,
				  uint32 AlphaMask = 0x0 );

	int    GetBPP()	      const { return m_BPP; } 
	uint32 GetRedMask()   const { return m_RedMask;   } 
	uint32 GetGreenMask() const { return m_GreenMask; } 
	uint32 GetBlueMask()  const { return m_BlueMask;  }
	uint32 GetAlphaMask() const { return m_AlphaMask; }

	uint32 BuildPixel( float Red, float Green, float Blue,
					   float Alpha = 0.0f ) const;

	uint32 BuildPixel( uint32 Red, uint32 Green, uint32 Blue,
					   uint32 Alpha = 0.0f ) const;

	uint32 BuildPixel( const CColor& Color ) const;

	uint32 GetRedBits( uint32 Pixel )   const;
	uint32 GetGreenBits( uint32 Pixel ) const;
	uint32 GetBlueBits( uint32 Pixel )  const;
	uint32 GetAlphaBits( uint32 Pixel ) const;

	uint32 ConvertStream( uchar* pDest, uint32 DestLength, const CPixelFormat& pfSource, uchar* pSrc, uint32 SrcLength ) const;

	friend inline int operator==( const CPixelFormat& pf1, const CPixelFormat& pf2 );
    friend inline int operator!=( const CPixelFormat& pf1, const CPixelFormat& pf2 );

private:
	int    m_BPP;           // Bits per pixel

    // bit mask required for each element (RGBA combos)
	uint32 m_RedMask;       // Red bit mask
	uint32 m_GreenMask;     // Green bit mask
	uint32 m_BlueMask;      // Blue bit mask
	uint32 m_AlphaMask;     // Alpha bitmask

    // number of bit shifts required for each element
	int m_RedShift;         // Red shift bits
	int m_GreenShift;       // Green shift bits
	int m_BlueShift;        // Blue shift bits
	int m_AlphaShift;       // Alpha shift bits

    // number of bits required for each element.
	int m_RedBits;          // Red bits
	int m_GreenBits;        // Green bits
	int m_BlueBits;         // Blue bits
	int m_AlphaBits;        // Alpha bits

	int GetShiftBits( uint32 Mask )
	{
		for( int shift = 0; ( ~Mask & 1 ) && shift < 32; Mask >>= 1 )
			shift ++;
		return shift;
	}

	int GetBits( uint32 Mask )
	{
		Mask >>= GetShiftBits( Mask );
		for( int bits = 0; ( Mask & 1 ) && bits < 32; Mask >>= 1 )
			bits ++;
		return bits;
	}
};

// Standard pixel formats used
const CPixelFormat RGB_555   = CPixelFormat( 16, 0x7C00,   0x3E0,  0x1F );
const CPixelFormat RGB_565   = CPixelFormat( 16, 0xF800,   0x7E0,  0x1F );
const CPixelFormat RGB_24    = CPixelFormat( 24, 0xFF0000, 0xFF00, 0xFF );
const CPixelFormat RGB_32    = CPixelFormat( 32, 0xFF0000, 0xFF00, 0xFF );
const CPixelFormat RGBA_1555 = CPixelFormat( 16, 0x7C00,   0x3E0,  0x1F, 0x8000 );
const CPixelFormat RGBA_24   = CPixelFormat( 24, 0xFF0000, 0xFF00, 0xFF, 0xFF000000 );
const CPixelFormat RGBA_32   = CPixelFormat( 32, 0xFF0000, 0xFF00, 0xFF, 0xFF000000 );
const CPixelFormat BGR_555   = CPixelFormat( 16, 0x1F,     0x3E0,  0x7C00 );
const CPixelFormat BGR_565   = CPixelFormat( 16, 0x1F,     0x7E0,  0xF800 );
const CPixelFormat BGR_24    = CPixelFormat( 24, 0xFF,     0xFF00, 0xFF0000 );
const CPixelFormat BGR_32    = CPixelFormat( 24, 0xFF,     0xFF00, 0xFF0000 );


/* ************************************************************************** */
// CPixelFormat::operator==( ... )
// - Pixel format compare operator
/* ************************************************************************** */
inline int operator==( const CPixelFormat& pf1, const CPixelFormat& pf2 ) 
{
	return	pf1.m_BPP		== pf2.m_BPP	   &&
			pf1.m_RedMask   == pf2.m_RedMask   &&
			pf1.m_GreenMask == pf2.m_GreenMask &&
			pf1.m_BlueMask  == pf2.m_BlueMask  &&
			pf1.m_AlphaMask == pf2.m_AlphaMask;
}

inline int operator!=( const CPixelFormat& pf1, const CPixelFormat& pf2 ) 
{
	return	pf1.m_BPP		!= pf2.m_BPP	   ||
			pf1.m_RedMask   != pf2.m_RedMask   ||
			pf1.m_GreenMask != pf2.m_GreenMask ||
			pf1.m_BlueMask  != pf2.m_BlueMask  ||
			pf1.m_AlphaMask != pf2.m_AlphaMask;
}

#endif